#!/usr/local/bin/perl -w

# management of the input
# requires two parameters: gold-standard-file and test-file
# both have the following format:
#    dieta 06041609-rttr-16k.txt O
#    dimagrante 06041609-rttr-16k.txt O
#    perla 06041609-rttr-16k.txt O
#    A 06041609-rttr-16k.txt B-ORG
#    ventidue 06041609-rttr-16k.txt I-ORG

# Calls the script levenstein.pl, which produces an alignment file called "appoggio.txt"
# In that file, the first two columns are the gold words and tags, and the last two columns are test words and tags

# We use arrays rather than hashmaps to ensure the output order is the same as the input order

my $line = "";
my $word = "";
my $file = "";
my $tag = "";

$argc = 0;
foreach (@ARGV) { $argc++; }

if ($argc!=2) {
    print "USO:\neval.perl richiede due parametri: gold-standard-file test-file\n";
    exit (1); }
else  {
    $gold = $ARGV[0];
    $test = $ARGV[1]; }

# Read the gold file into a multidimensional array
open(GOLD, "< $gold") or die "Impossibile aprire il file $gold";
@raw_gold_data = <GOLD>;

my $currgoldfile = "";
my $goldsentence = "";
my $goldtags = "";
my $goldfilecount = 0;
my @goldsentences;
my @goldtagset;

foreach $line (@raw_gold_data) {
  chomp($line);
  ($word,$file,$tag) = split(' ',$line);
  if ($currgoldfile eq "") { $goldsentence = $word; $goldtags = $tag; } # Setup for the very first line read
  if ($file eq $currgoldfile) { # If the second column indicates this is the same file as before
      $goldsentence = $goldsentence . " " . $word; # Then append the word to the "sentence", where an entire transcript is one "sentence"
      $goldtags = $goldtags . " " . $tag; }        # Do the same for the tag
  else { if ($currgoldfile ne "") { # If the second column indicates we have changed to a new file
                                    # Then reset to the start of the next row in the array
             push @goldsentences, $goldsentence; push @goldtagset, $goldtags; $goldsentence = ""; }
         $goldfilecount++; $currgoldfile = $file; }
}
push @goldsentences, $goldsentence;
push @goldtagset, $goldtags;
$goldfilecount++;

# Repeat the above procedure for the test words and tags
open(TEST, "< $test") or die "Impossibile aprire il file $test";
@raw_test_data = <TEST>;

my $currtestfile = "";
my $testsentence = "";
my $testtags = "";
my @testsentences;
my @testtagset;

foreach $line (@raw_test_data) {
  chomp($line);
  ($word,$file,$tag) = split(' ',$line);
  if ($currtestfile eq "") { $testsentence = $word; $testtags = $tag; }
  if ($file eq $currtestfile) {
      $testsentence = $testsentence . " " . $word;
      $testtags = $testtags . " " . $tag; }
  else { if ($currtestfile ne "") {
             push @testsentences, $testsentence; push @testtagset, $testtags; $testsentence = ""; }
         $currtestfile = $file; }
}
push @testsentences, $testsentence;
push @testtagset, $testtags;


# Create a file with all the gold and test tags in order so that levenstein.pl can access them
open(GOLDTAGS, "> goldtags.txt") or die "Impossibile aprire il file goldtags.txt";
foreach $line (@goldtagset) { print GOLDTAGS "$line\n"; }
close(GOLDTAGS);
open(TESTTAGS, "> testtags.txt") or die "Impossibile aprire il file testtags.txt";
foreach $line (@testtagset) { print TESTTAGS "$line\n"; }
close(TESTTAGS);

print "#Files read: $goldfilecount\n";



# Run the levenstein.pl script to produce the alignment
my $file_di_appoggio = "appoggio.txt";
my $gline = "";
my $tline = "";
if (-e "levenstein.pl") {
    open(LEVTXT,"|perl levenstein.pl -2 -e > $file_di_appoggio");
        for ($i = 0; $i < $goldfilecount; $i++) {
	$gline = lc(@goldsentences[$i]);
	$tline = lc(@testsentences[$i]);
	# Send all the transcript "sentences" in pairs to the levenstein script for alignment
                print LEVTXT "$tline\n$gline\n";
        }
    close(LEVTXT); }
else { print "\n\nError: impossibile aprire levenstein\n"; }


# Show some basic statistics when we are done
print "\n\nASR evaluation\n------------------------\n";
if (-e "levenstein.pl") {
    open(FILE_LEV, "< $file_di_appoggio") or die "Impossibile aprire il file di appoggio";
    while (!eof(FILE_LEV)) {
        $eval_lev = <FILE_LEV>;
	    if ($eval_lev=~/String Recognition Rate/i) {
            print "$eval_lev"; }
	    if ($eval_lev=~/Unit Accuracy/i) {
	        print "$eval_lev"; }
	    if ($eval_lev=~/Errors/i) {
            print "$eval_lev\n"; } }
    close(FILE_LEV); }
else { print "Error: $file_di_appoggio e' vuoto\n"; }


unlink("goldtags.txt");
unlink("testtags.txt");



